@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 #
 # Copyright (c) 2009- Spyder Kernels Contributors
 #
@@ -8,61 +9,59 @@
 
 import ast
 import bdb
+import builtins
+from contextlib import contextmanager
 import logging
 import os
 import sys
 import traceback
+import threading
 from collections import namedtuple
+from functools import lru_cache
 
 from IPython.core.autocall import ZMQExitAutocall
-from IPython.core.getipython import get_ipython
 from IPython.core.debugger import Pdb as ipyPdb
+from IPython.core.inputtransformer2 import TransformerManager
 
+import spyder_kernels
+from spyder_kernels.comms.commbase import stacksummary_to_json
 from spyder_kernels.comms.frontendcomm import CommError, frontend_request
-from spyder_kernels.customize.utils import path_is_library
-from spyder_kernels.py3compat import TimeoutError, PY2, _print, isidentifier
-
-if not PY2:
-    from IPython.core.inputtransformer2 import TransformerManager
-    import builtins
-    basestring = (str,)
-else:
-    import __builtin__ as builtins
-    from IPython.core.inputsplitter import IPythonInputSplitter as TransformerManager
+from spyder_kernels.customize.utils import (
+    path_is_library, capture_last_Expr, exec_encapsulate_locals
+)
 
 
 logger = logging.getLogger(__name__)
 
 
-class DebugWrapper(object):
+class DebugWrapper:
     """
-    Notifies the frontend when debuggging starts/stops
+    Notifies the frontend when debugging starts/stops
     """
     def __init__(self, pdb_obj):
         self.pdb_obj = pdb_obj
+        self._cleanup = True
 
     def __enter__(self):
         """
         Debugging starts.
         """
-        self.pdb_obj._frontend_notified = True
-        try:
-            frontend_request(blocking=True).set_debug_state(True)
-        except (CommError, TimeoutError):
-            logger.debug("Could not send debugging state to the frontend.")
+        shell = self.pdb_obj.shell
+        if shell.pdb_session == self.pdb_obj:
+            self._cleanup = False
+        else:
+            shell.add_pdb_session(self.pdb_obj)
+            self._cleanup = True
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         """
         Debugging ends.
         """
-        self.pdb_obj._frontend_notified = False
-        try:
-            frontend_request(blocking=True).set_debug_state(False)
-        except (CommError, TimeoutError):
-            logger.debug("Could not send debugging state to the frontend.")
+        if self._cleanup:
+            self.pdb_obj.shell.remove_pdb_session(self.pdb_obj)
 
 
-class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
+class SpyderPdb(ipyPdb):
     """
     Extends Pdb to add features:
 
@@ -73,9 +72,6 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
      - Add completion to non-command code.
     """
 
-    send_initial_notification = True
-    starting = True
-
     def __init__(self, completekey='tab', stdin=None, stdout=None,
                  skip=None, nosigint=False):
         """Init Pdb."""
@@ -85,12 +81,47 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
         self.pdb_ignore_lib = False
         self.pdb_execute_events = False
         self.pdb_use_exclamation_mark = False
+        self.pdb_publish_stack = False
         self._exclamation_warning_printed = False
         self.pdb_stop_first_line = True
         self._disable_next_stack_entry = False
         super(SpyderPdb, self).__init__()
-        self._pdb_breaking = False
-        self._frontend_notified = False
+
+        # content of tuple: (filename, line number)
+        self._previous_step = None
+
+        # Don't report hidden frames for IPython 7.24+. This attribute
+        # has no effect in previous versions.
+        self.report_skipped = False
+
+        # Keep track of remote filename
+        self.remote_filename = None
+
+        # Needed to know which namespace to show (user or current frame)
+        # Line received from the frontend
+        self._cmd_input_line = None
+
+        # Disable sigint so we can do it ourselves
+        self.nosigint = True
+
+        # Keep track of interrupting state to avoid several interruptions
+        self.interrupting = False
+
+        # Should the frontend force go to the current line?
+        self._request_where = False
+
+        # Turn off IPython's debugger skip funcionality by default because
+        # it makes our debugger quite slow. It's also important to remark
+        # that this functionality doesn't do anything on its own. Users
+        # need to mark what frames they want to skip for it to be useful.
+        # So, we hope that knowledgeable users will find that they need to
+        # enable it in Spyder.
+        # Fixes spyder-ide/spyder#20639.
+        self._predicates["debuggerskip"] = False
+
+        # Save seen files inodes
+        self._canonic_inode_to_filename = {}
+        self._canonic_filename_to_inode = {}
 
     # --- Methods overriden for code execution
     def print_exclamation_warning(self):
@@ -103,10 +134,6 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
     def default(self, line):
         """
         Default way of running pdb statment.
-
-        The only difference with Pdb.default is that if line contains multiple
-        statments, the code will be compiled with 'exec'. It will not print the
-        result but will run without failing.
         """
         execute_events = self.pdb_execute_events
         if line[:1] == '!':
@@ -115,33 +142,29 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
             self.print_exclamation_warning()
             self.error("Unknown command '" + line.split()[0] + "'")
             return
-        # Disallow the use of %debug magic in the debugger
-        if line.startswith("%debug"):
-            self.error("Please don't use '%debug' in the debugger.\n"
-                       "For a recursive debugger, use the pdb 'debug'"
-                       " command instead")
-            return
-        locals = self.curframe_locals
 
-        # This is necessary to allow running comprehensions with the
-        # frame locals. It also fallbacks to the right globals if the
-        # user wants to work with them instead.
-        # See spyder-ide/spyder#13909.
-        if not 'globals()' in line:
-            ns = self.curframe.f_globals.copy()
-            ns.update(locals)
-        else:
-            ns = self.curframe.f_globals
+        # Replace %debug magic in the debugger
+        if line.startswith("%debug") or line.startswith("%%debug"):
+            cmd, arg, _ = self.parseline(line.lstrip("%"))
+            if cmd == "debug":
+                return self.do_debug(arg)
+
+        local_ns = self.curframe_locals
+        global_ns = self.curframe.f_globals
 
         if self.pdb_use_exclamation_mark:
             # Find pdb commands executed without !
             cmd, arg, line = self.parseline(line)
             if cmd:
                 cmd_in_namespace = (
-                    cmd in ns or cmd in builtins.__dict__)
+                    cmd in global_ns
+                    or cmd in local_ns
+                    or cmd in builtins.__dict__
+                )
                 # Special case for quit and exit
                 if cmd in ("quit", "exit"):
-                    if cmd in ns and isinstance(ns[cmd], ZMQExitAutocall):
+                    if cmd in global_ns and isinstance(
+                            global_ns[cmd], ZMQExitAutocall):
                         # Use the pdb call
                         cmd_in_namespace = False
                 cmd_func = getattr(self, 'do_' + cmd, None)
@@ -165,12 +188,8 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
                         # The pdb command is masked by something
                         self.print_exclamation_warning()
         try:
+            is_magic = line.startswith("%")
             line = TransformerManager().transform_cell(line)
-            try:
-                code = compile(line + '\n', '<stdin>', 'single')
-            except SyntaxError:
-                # support multiline statments
-                code = compile(line + '\n', '<stdin>', 'exec')
             save_stdout = sys.stdout
             save_stdin = sys.stdin
             save_displayhook = sys.displayhook
@@ -179,93 +198,143 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
                 sys.stdout = self.stdout
                 sys.displayhook = self.displayhook
                 if execute_events:
-                     get_ipython().events.trigger('pre_execute')
-                exec(code, ns, locals)
-                if execute_events:
-                     get_ipython().events.trigger('post_execute')
+                     self.shell.events.trigger('pre_execute')
+
+                code_ast = ast.parse(line)
+
+                if line.rstrip()[-1:] == ";":
+                    # Supress output with ;
+                    capture_last_expression = False
+                else:
+                    code_ast, capture_last_expression = capture_last_Expr(
+                        code_ast, "_spyderpdb_out", global_ns)
+
+                if is_magic:
+                    # Magics like runcell use and modify local_ns.
+                    # But the locals() dict can not be directly modified when
+                    # encapsulated. Therefore they must encapsulate the locals
+                    # themselves (see code_runner.py).
+                    exec(compile(code_ast, "<stdin>", "exec"), global_ns, local_ns)
+                else:
+                    exec_encapsulate_locals(code_ast, global_ns, local_ns)
+
+                if capture_last_expression:
+                    out = global_ns.pop("_spyderpdb_out", None)
+                    if out is not None:
+                        sys.stdout.flush()
+                        sys.stderr.flush()
+                        try:
+                            frontend_request(blocking=False).show_pdb_output(
+                                repr(out))
+                        except (CommError, TimeoutError):
+                            # Fallback
+                            print("pdb out> ", repr(out))
+
             finally:
+                if execute_events:
+                     self.shell.events.trigger('post_execute')
                 sys.stdout = save_stdout
                 sys.stdin = save_stdin
                 sys.displayhook = save_displayhook
         except BaseException:
-            if PY2:
-                t, v = sys.exc_info()[:2]
-                if type(t) == type(''):
-                    exc_type_name = t
-                else: exc_type_name = t.__name__
-                print >>self.stdout, '***', exc_type_name + ':', v
-            else:
-                exc_info = sys.exc_info()[:2]
-                self.error(
-                    traceback.format_exception_only(*exc_info)[-1].strip())
+            exc_info = sys.exc_info()[:2]
+            self.error(
+                traceback.format_exception_only(*exc_info)[-1].strip())
 
     # --- Methods overriden for signal handling
-    def sigint_handler(self, signum, frame):
-        """
-        Handle a sigint signal. Break on the frame above this one.
-
-        This method is not present in python2 so this won't be called there.
-        """
-        if self.allow_kbdint:
-            raise KeyboardInterrupt
+    def interrupt(self):
+        """Stop debugger on next instruction."""
+        self.interrupting = True
         self.message("\nProgram interrupted. (Use 'cont' to resume).")
-        # avoid stopping in set_trace
-        sys.settrace(None)
-        self._pdb_breaking = True
         self.set_step()
-        self.set_trace(sys._getframe())
+
+    def set_trace(self, frame=None):
+        """Register that debugger is tracing."""
+        self.shell.add_pdb_session(self)
+        super(SpyderPdb, self).set_trace(frame)
+
+    def set_quit(self):
+        """Register that debugger is not tracing."""
+        self.shell.remove_pdb_session(self)
+        super(SpyderPdb, self).set_quit()
 
     def interaction(self, frame, traceback):
         """
         Called when a user interaction is required.
-
-        If this is from sigint, break on the upper frame.
-        If the frame is in spydercustomize.py, quit.
-        Notifies spyder and print current code.
-
         """
-        if self._pdb_breaking:
-            self._pdb_breaking = False
-            if frame and frame.f_back:
-                return self.interaction(frame.f_back, traceback)
+        with DebugWrapper(self):
+            # Wrapp in case the frontend was not notified, e.g. postmortem
+            return super(SpyderPdb, self).interaction(
+                frame, traceback)
 
-        self.setup(frame, traceback)
-        self.print_stack_entry(self.stack[self.curindex])
-        if self._frontend_notified:
-            self._cmdloop()
-        else:
-            with DebugWrapper(self):
-                self._cmdloop()
-        self.forget()
-
-    def print_stack_entry(self, frame_lineno, prompt_prefix='\n-> ',
-                          context=None):
+    def print_stack_entry(self, *args, **kwargs):
         """Disable printing stack entry if requested."""
         if self._disable_next_stack_entry:
             self._disable_next_stack_entry = False
             return
-        return super(SpyderPdb, self).print_stack_entry(
-            frame_lineno, prompt_prefix, context)
+        return super().print_stack_entry(*args, **kwargs)
 
     # --- Methods overriden for skipping libraries
     def stop_here(self, frame):
         """Check if pdb should stop here."""
-        if (frame is not None
-                and frame.f_locals.get(
-                    "__tracebackhide__", False) == "__pdb_exit__"):
+        # Never stop if we are continuing unless there is a breakpoint
+        if self.stopframe == self.botframe and self.stoplineno == -1:
+            return False
+        if self.continue_if_has_breakpoints and self.should_continue(frame):
+            self.set_continue()
+            return False
+        if (
+            frame is not None
+            and "__tracebackhide__" in frame.f_locals
+            and frame.f_locals["__tracebackhide__"] == "__pdb_exit__"
+        ):
             self.onecmd('exit')
             return False
 
-        if not super(SpyderPdb, self).stop_here(frame):
+        if not super().stop_here(frame):
             return False
+        if frame is self.stopframe:
+            return True
         filename = frame.f_code.co_filename
         if filename.startswith('<'):
             # This is not a file
             return True
         if self.pdb_ignore_lib and path_is_library(filename):
             return False
+        if self.skip_hidden and os.path.dirname(spyder_kernels.__file__) in filename:
+            # This is spyder-kernels internals
+            return False
         return True
 
+    def should_continue(self, frame):
+        """
+        Jump to first breakpoint if needed.
+
+        Fixes spyder-ide/spyder#2034
+        """
+
+        if not self.continue_if_has_breakpoints:
+            # This was disabled
+            return False
+        self.continue_if_has_breakpoints = False
+
+        # Get all breakpoints for the file we're going to debug
+        if not frame:
+            # We are not debugging, return. Solves spyder-ide/spyder#10290
+            return False
+
+        lineno = frame.f_lineno
+        breaks = self.get_file_breaks(frame.f_code.co_filename)
+
+        # Do 'continue' if the first breakpoint is *not* placed
+        # where the debugger is going to land.
+        # Fixes spyder-ide/spyder#4681
+        if self.pdb_stop_first_line:
+            return breaks and lineno < breaks[0]
+
+        # The breakpoint could be in another file.
+        return not (breaks and lineno >= breaks[0])
+
     def do_where(self, arg):
         """w(here)
         Print a stack trace, with the most recent frame at the bottom.
@@ -274,8 +343,8 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
 
         Take a number as argument as an (optional) number of context line to
         print"""
-        super(SpyderPdb, self).do_where(arg)
-        frontend_request().do_where()
+        self._request_where = True
+        return super(SpyderPdb, self).do_where(arg)
 
     do_w = do_where
 
@@ -331,7 +400,7 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
             if not text or text[0] == '.':
                 return False
             # We want to keep value.subvalue
-            return isidentifier(text.replace('.', ''))
+            return text.replace('.', '').isidentifier()
 
         while text and not is_name_or_composed(text):
             text = text[1:]
@@ -344,7 +413,6 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
         cursor_start = cursor_pos - len(text)
 
         if ipython_do_complete:
-            kernel = get_ipython().kernel
             # Make complete call with current frame
             if self.curframe:
                 if self.curframe_locals:
@@ -353,10 +421,10 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
                                   self.curframe.f_globals)
                 else:
                     frame = self.curframe
-                kernel.shell.set_completer_frame(frame)
-            result = kernel._do_complete(code, cursor_pos)
+                self.shell.set_completer_frame(frame)
+            result = self.shell.kernel._do_complete(code, cursor_pos)
             # Reset frame
-            kernel.shell.set_completer_frame()
+            self.shell.set_completer_frame()
             # If there is no Pdb results to merge, return the result
             if not compfunc:
                 return result
@@ -434,7 +502,7 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
                 if not text or text[0] == '.':
                     return False
                 # We want to keep value.subvalue
-                return isidentifier(text.replace('.', ''))
+                return text.replace('.', '').isidentifier()
 
             while text and not is_name_or_composed(text):
                 text = text[1:]
@@ -452,7 +520,6 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
                 'status': 'ok'
                 }
 
-        kernel = get_ipython().kernel
         # Make complete call with current frame
         if self.curframe:
             if self.curframe_locals:
@@ -461,29 +528,18 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
                               self.curframe.f_globals)
             else:
                 frame = self.curframe
-            kernel.shell.set_completer_frame(frame)
-        result = kernel._do_complete(code, cursor_pos)
+            self.shell.set_completer_frame(frame)
+        result = self.shell.kernel._do_complete(code, cursor_pos)
         # Reset frame
-        kernel.shell.set_completer_frame()
+        self.shell.set_completer_frame()
         return result
 
     # --- Methods overriden by us for Spyder integration
-    def preloop(self):
-        """Ask Spyder for breakpoints before the first prompt is created."""
-        try:
-            pdb_settings = frontend_request().get_pdb_settings()
-            self.pdb_ignore_lib = pdb_settings['pdb_ignore_lib']
-            self.pdb_execute_events = pdb_settings['pdb_execute_events']
-            self.pdb_use_exclamation_mark = pdb_settings[
-                'pdb_use_exclamation_mark']
-            self.pdb_stop_first_line = pdb_settings['pdb_stop_first_line']
-            if self.starting:
-                self.set_spyder_breakpoints(pdb_settings['breakpoints'])
-            if self.send_initial_notification:
-                self.notify_spyder()
-        except (CommError, TimeoutError):
-            logger.debug("Could not get breakpoints from the frontend.")
-        super(SpyderPdb, self).preloop()
+    def postloop(self):
+        # postloop() is called when the debugger’s input prompt exists. Reset
+        # _previous_step so that get_pdb_state() actually notifies Spyder
+        # about a changed frame the next the input prompt is entered again.
+        self._previous_step = None
 
     def set_continue(self):
         """
@@ -495,14 +551,6 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
         # Don't stop except at breakpoints or when finished
         self._set_stopinfo(self.botframe, None, -1)
 
-    def reset(self):
-        """
-        Register Pdb session after reset.
-        """
-        super(SpyderPdb, self).reset()
-        kernel = get_ipython().kernel
-        kernel._register_pdb_session(self)
-
     def do_debug(self, arg):
         """
         Debug code
@@ -511,9 +559,43 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
         argument (which is an arbitrary expression or statement to be
         executed in the current environment).
         """
-        super(SpyderPdb, self).do_debug(arg)
-        kernel = get_ipython().kernel
-        kernel._register_pdb_session(self)
+        with self.recursive_debugger() as debugger:
+            self.message("Entering recursive debugger")
+            try:
+                global_ns = self.curframe.f_globals
+                local_ns = self.curframe_locals
+                return sys.call_tracing(debugger.run, (arg, global_ns, local_ns))
+            except Exception:
+                exc_info = sys.exc_info()[:2]
+                self.error(
+                    traceback.format_exception_only(*exc_info)[-1].strip())
+            finally:
+                self.message("Leaving recursive debugger")
+
+    @contextmanager
+    def recursive_debugger(self):
+        """Get a recursive debugger."""
+        # Save and restore tracing function
+        trace_function = sys.gettrace()
+        sys.settrace(None)
+
+        # Create child debugger
+        debugger = self.__class__(
+            completekey=self.completekey,
+            stdin=self.stdin, stdout=self.stdout)
+        debugger.prompt = "(%s) " % self.prompt.strip()
+        try:
+            yield debugger
+        finally:
+            # Reset parent debugger
+            sys.settrace(trace_function)
+            self.lastcmd = debugger.lastcmd
+
+            # Reset _previous_step so that get_pdb_state() notifies Spyder about
+            # a changed debugger position. The reset is required because the
+            # recursive debugger might change the position, but the parent
+            # debugger (self) is not aware of this.
+            self._previous_step = None
 
     def user_return(self, frame, return_value):
         """This function is called when a return trap is set here."""
@@ -528,6 +610,7 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
 
     def _cmdloop(self):
         """Modifies the error text."""
+        self.interrupting = False
         while True:
             try:
                 # keyboard interrupts allow for an easy way to cancel
@@ -537,9 +620,134 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
                 self.allow_kbdint = False
                 break
             except KeyboardInterrupt:
-                _print("--KeyboardInterrupt--\n"
-                       "For copying text while debugging, use Ctrl+Shift+C",
-                       file=self.stdout)
+                print("--KeyboardInterrupt--\n"
+                      "For copying text while debugging, use Ctrl+Shift+C",
+                      file=self.stdout)
+
+    @lru_cache
+    def canonic(self, filename):
+        """
+        Return canonical form of filename.
+
+        In some case two path can point to the same file. For this reason
+        os.path.samefile uses os.stat. Here we normalise the path with os.stat
+        so a single path is returned for the same file.
+
+        see: https://docs.python.org/3/library/os.path.html#os.path.samefile
+        note: os.stat can be slow on windows so call it once per file.
+        """
+        if filename == "<" + filename[1:-1] + ">":
+            return filename
+
+        filename = super().canonic(filename)
+
+        if filename in self._canonic_filename_to_inode:
+            inode = self._canonic_filename_to_inode[filename]
+        else:
+            try:
+                stat = os.stat(filename)
+            except OSError:
+                self._canonic_filename_to_inode[filename] = None
+                return filename
+
+            inode = (stat.st_dev, stat.st_ino)
+            if stat.st_ino == 0:
+                inode = None
+            self._canonic_filename_to_inode[filename] = inode
+            if inode is not None and inode not in self._canonic_inode_to_filename:
+                # First time this inode is seen
+                self._canonic_inode_to_filename[inode] = filename
+
+        if inode is None:
+            return filename
+        return self._canonic_inode_to_filename[inode]
+
+
+    def do_exitdb(self, arg):
+        """Exit the debugger"""
+        self._set_stopinfo(self.botframe, None, -1)
+        sys.settrace(None)
+        frame = sys._getframe().f_back
+        while frame and frame is not self.botframe:
+            del frame.f_trace
+            frame = frame.f_back
+        return 1
+
+    def cmdloop(self, intro=None):
+        """
+        Repeatedly issue a prompt, accept input, parse an initial prefix
+        off the received input, and dispatch to action methods, passing them
+        the remainder of the line as argument.
+        """
+        self.preloop()
+        if intro is not None:
+            self.intro = intro
+        if self.intro:
+            self.stdout.write(str(self.intro)+"\n")
+        stop = None
+        while not stop:
+            if self.cmdqueue:
+                # Anything available in cmdqueue is a Pdb command, so we need
+                # to process it as such.
+                # Fixes spyder-ide/spyder#22500
+                line = (
+                    "!" if self.pdb_use_exclamation_mark else ""
+                ) + self.cmdqueue.pop(0)
+            else:
+                try:
+                    line = self.cmd_input(self.prompt)
+                except EOFError:
+                    line = 'EOF'
+            line = self.precmd(line)
+            stop = self.onecmd(line)
+            stop = self.postcmd(stop, line)
+        self.postloop()
+
+    def cmd_input(self, prompt=''):
+        """
+        Get input from frontend. Blocks until return
+        """
+        kernel = self.shell.kernel
+        # Only works if the comm is open
+        if not kernel.frontend_comm.is_open():
+            return input(prompt)
+
+        # Flush output before making the request.
+        sys.stderr.flush()
+        sys.stdout.flush()
+        sys.__stderr__.flush()
+        sys.__stdout__.flush()
+
+        # Send the input request.
+        self._cmd_input_line = None
+        kernel.frontend_call(display_error=True).pdb_input(
+            prompt, state=self.get_pdb_state())
+
+        # Allow GUI event loop to update
+        is_main_thread = (
+            threading.current_thread() is threading.main_thread())
+
+        # Get input by running eventloop
+        if is_main_thread and kernel.eventloop:
+            while self._cmd_input_line is None:
+                eventloop = kernel.eventloop
+                # Check if the current backend is Tk on Windows
+                # to let GUI update.
+                # See spyder-ide/spyder#17523
+                if (eventloop and hasattr(kernel, "app_wrapper") and
+                        os.name == "nt"):
+                    kernel.app_wrapper.app.update()
+                elif eventloop:
+                    eventloop(kernel)
+                else:
+                    break
+
+        # Get input by blocking
+        if self._cmd_input_line is None:
+            kernel.frontend_comm.wait_until(
+                lambda: self._cmd_input_line is not None)
+
+        return self._cmd_input_line
 
     def precmd(self, line):
         """
@@ -559,44 +767,11 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
         return line
 
     def postcmd(self, stop, line):
-        """
-        Notify spyder on any pdb command.
-        """
-        self.notify_spyder(self.curframe)
-        return super(SpyderPdb, self).postcmd(stop, line)
-
-    if PY2:
-        def break_here(self, frame):
-            """
-            Breakpoints don't work for files with non-ascii chars in Python 2
-
-            Fixes Issue 1484
-            """
-            from bdb import effective
-            filename = self.canonic(frame.f_code.co_filename)
-            try:
-                filename = unicode(filename, "utf-8")
-            except TypeError:
-                pass
-            if filename not in self.breaks:
-                return False
-            lineno = frame.f_lineno
-            if lineno not in self.breaks[filename]:
-                # The line itself has no breakpoint, but maybe the line is the
-                # first line of a function with breakpoint set by function name
-                lineno = frame.f_code.co_firstlineno
-                if lineno not in self.breaks[filename]:
-                    return False
-
-            # flag says ok to delete temp. bp
-            (bp, flag) = effective(filename, lineno, frame)
-            if bp:
-                self.currentbp = bp.number
-                if (flag and bp.temporary):
-                    self.do_clear(str(bp.number))
-                return True
-            else:
-                return False
+        """Hook method executed just after a command dispatch is finished."""
+        # Flush in case the command produced output on underlying outputs
+        sys.__stderr__.flush()
+        sys.__stdout__.flush()
+        return stop
 
     # --- Methods defined by us for Spyder integration
     def set_spyder_breakpoints(self, breakpoints):
@@ -620,76 +795,62 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
                     # The file is not readable
                     pass
 
-        # Jump to first breakpoint.
-        # Fixes issue 2034
-        if self.starting:
-            # Only run this after a Pdb session is created
-            self.starting = False
+    breakpoints = property(fset=set_spyder_breakpoints)
 
-            # Get all breakpoints for the file we're going to debug
-            frame = self.curframe
-            if not frame:
-                # We are not debugging, return. Solves #10290
-                return
-            lineno = frame.f_lineno
-            breaks = self.get_file_breaks(frame.f_code.co_filename)
+    def get_pdb_state(self):
+        """
+        Send debugger state (frame position) to the frontend.
 
-            # Do 'continue' if the first breakpoint is *not* placed
-            # where the debugger is going to land.
-            # Fixes issue 4681
-            if self.pdb_stop_first_line:
-                do_continue = (
-                    self.continue_if_has_breakpoints
-                    and breaks
-                    and lineno < breaks[0])
-            else:
-                # The breakpoint could be in another file.
-                do_continue = (
-                    self.continue_if_has_breakpoints
-                    and not (breaks and lineno >= breaks[0]))
+        The state is only sent if it has changed since the last update.
+        """
+        state = self.shell.kernel.get_state()
 
-            if do_continue:
-                try:
-                    if self.pdb_use_exclamation_mark:
-                        cont_cmd = '!continue'
-                    else:
-                        cont_cmd = 'continue'
-                    frontend_request(blocking=False).pdb_execute(cont_cmd)
-                except (CommError, TimeoutError):
-                    logger.debug(
-                        "Could not send a Pdb continue call to the frontend.")
-
-    def notify_spyder(self, frame=None):
-        """Send kernel state to the frontend."""
+        frame = self.curframe
         if frame is None:
-            frame = self.curframe
+            self._previous_step = None
+            return state
 
-        if frame is None:
-            return
-
-        kernel = get_ipython().kernel
+        if self._request_where:
+            self._request_where = False
+            state["do_where"] = True
 
         # Get filename and line number of the current frame
         fname = self.canonic(frame.f_code.co_filename)
-        if PY2:
-            try:
-                fname = unicode(fname, "utf-8")
-            except TypeError:
-                pass
+        if fname == self.mainpyfile and self.remote_filename is not None:
+            fname = self.remote_filename
         lineno = frame.f_lineno
 
+        if self._previous_step == (fname, lineno):
+            # Do not update state if not needed
+            return state
+
         # Set step of the current frame (if any)
         step = {}
-        if isinstance(fname, basestring) and isinstance(lineno, int):
+        self._previous_step = None
+        if isinstance(fname, str) and isinstance(lineno, int):
             step = dict(fname=fname, lineno=lineno)
+            self._previous_step = (fname, lineno)
 
-        # Publish Pdb state so we can update the Variable Explorer
-        # and the Editor on the Spyder side
-        kernel._pdb_step = step
-        try:
-            kernel.publish_pdb_state()
-        except (CommError, TimeoutError):
-            logger.debug("Could not send Pdb state to the frontend.")
+        state['step'] = step
+
+        if self.pdb_publish_stack:
+            # Publish Pdb stack so we can update the Debugger plugin on Spyder
+            pdb_stack = stacksummary_to_json(
+                traceback.StackSummary.extract(self.stack)
+            )
+            pdb_index = self.curindex
+            skip_hidden = getattr(self, 'skip_hidden', False)
+
+            if skip_hidden:
+                # Filter out the hidden frames
+                hidden = self.hidden_frames(self.stack)
+                pdb_stack = [f for f, h in zip(pdb_stack, hidden) if not h]
+                # Adjust the index
+                pdb_index -= sum([bool(i) for i in hidden[:pdb_index]])
+
+            state['stack'] = (pdb_stack, pdb_index)
+
+        return state
 
     def run(self, cmd, globals=None, locals=None):
         """Debug a statement executed via the exec() function.
@@ -715,43 +876,8 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
         with DebugWrapper(self):
             super(SpyderPdb, self).runcall(*args, **kwds)
 
-
-def enter_debugger(filename, continue_if_has_breakpoints, code_format):
-    """Enter debugger. Code format should be a format that accept filename."""
-    kernel = get_ipython().kernel
-    recursive = kernel.is_debugging()
-    if recursive:
-        parent_debugger = kernel._pdb_obj
-        sys.settrace(None)
-        globals = parent_debugger.curframe.f_globals
-        locals = parent_debugger.curframe_locals
-        # Create child debugger
-        debugger = SpyderPdb(
-            completekey=parent_debugger.completekey,
-            stdin=parent_debugger.stdin, stdout=parent_debugger.stdout)
-        debugger.use_rawinput = parent_debugger.use_rawinput
-        debugger.prompt = "(%s) " % parent_debugger.prompt.strip()
-    else:
-        debugger = SpyderPdb()
-
-    filename = debugger.canonic(filename)
-    debugger._wait_for_mainpyfile = True
-    debugger.mainpyfile = filename
-    debugger.continue_if_has_breakpoints = continue_if_has_breakpoints
-    debugger._user_requested_quit = False
-
-    if os.name == 'nt':
-        filename = filename.replace('\\', '/')
-
-    code = code_format.format(repr(filename))
-
-    if recursive:
-        # Enter recursive debugger
-        sys.call_tracing(debugger.run, (code, globals, locals))
-        # Reset parent debugger
-        sys.settrace(parent_debugger.trace_dispatch)
-        parent_debugger.lastcmd = debugger.lastcmd
-        kernel._register_pdb_session(parent_debugger)
-    else:
-        # The breakpoint might not be in the cell
-        debugger.run(code)
+    def set_remote_filename(self, filename):
+        """Set remote filename to signal Spyder on mainpyfile."""
+        self.remote_filename = filename
+        self.mainpyfile = self.canonic(filename)
+        self._wait_for_mainpyfile = True
